package com.bonus.sgzb.material.mapper;

import com.bonus.sgzb.base.api.domain.SltAgreementInfo;
import com.bonus.sgzb.material.domain.AgreementInfo;
import com.bonus.sgzb.material.domain.CalMonthlyBean;
import com.bonus.sgzb.material.domain.ProjectMonthCosts;
import com.bonus.sgzb.material.domain.ProjectMonthDetail;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * @description  综合查询--维修记录查询
 * @author hay
 * @date   2024/2/26 14:19
 */
@Mapper
public interface CalMonthlyMapper {

    /**
     * 综合查询--维修记录列表
     * @param bean
     * @return List<RepairRecord>
     */
    int insertCalcRecord(CalMonthlyBean bean);

    List<CalMonthlyBean> getCalcRecords(CalMonthlyBean bean);

    int deleteCalcRecord(CalMonthlyBean bean);

    int deleteMonthlyDetail(CalMonthlyBean bean);

    int deleteMonthlyCosts(CalMonthlyBean bean);

    int insertProMonCosts(ProjectMonthCosts projectMonthCosts);

    int insertProjectMonthDetail(ProjectMonthDetail projectMonthDetail);

    int updateProMonCosts(ProjectMonthCosts pmcId);

    List<SltAgreementInfo> getMonthCosts(AgreementInfo agreementInfo);

    List<SltAgreementInfo> getMonthDetails(SltAgreementInfo monthCost);
}
